from django import forms
from django.contrib.admin.widgets import ForeignKeyRawIdWidget
from django.core.exceptions import ValidationError
from django.db import models
from django.urls import NoReverseMatch, reverse
from django.utils.text import Truncator
from django.utils.translation import ugettext_lazy as _

from cabinet.base_admin import folder_choices
from cabinet.models import Folder


class UploadForm(forms.Form):
    folder = forms.ModelChoiceField(label=_("folder"), queryset=Folder.objects.all())
    file = forms.FileField(label=_("file"))

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields["folder"].choices = list(folder_choices())


class CabinetFileRawIdWidget(ForeignKeyRawIdWidget):
    template_name = "admin/cabinet/cabinet_file_raw_id_widget.html"

    class Media:
        css = {"all": ["cabinet/inline-upload.css"]}
        js = ["cabinet/inline-upload.js"]

    def get_context(self, name, value, attrs):
        context = super().get_context(name, value, attrs)
        instance = getattr(self, "instance", None)
        context["cabinet"] = {
            "upload_form": UploadForm(
                prefix="cu-{}".format(id(self)),
                initial={"folder": instance and instance.folder_id},
            ),
            "instance": instance,
        }
        if instance:
            context["related_url"] += "&folder__id__exact={}".format(instance.folder_id)
        return context

    def label_and_url_for_value(self, value):
        # Copied from django/contrib/admin/widgets.py with the addition of
        # saving the obj as self.instance
        key = self.rel.get_related_field().name
        try:
            obj = self.rel.model._default_manager.using(self.db).get(**{key: value})
        except (ValueError, self.rel.model.DoesNotExist, ValidationError):
            obj = None
            return "", ""

        try:
            url = reverse(
                "%s:%s_%s_change"
                % (
                    self.admin_site.name,
                    obj._meta.app_label,
                    obj._meta.object_name.lower(),
                ),
                args=(obj.pk,),
            )
        except NoReverseMatch:
            url = ""  # Admin not registered for target model.

        self.instance = obj
        return Truncator(obj).words(14, truncate="..."), url


class CabinetForeignKey(models.ForeignKey):
    def deconstruct(self):
        name, path, args, kwargs = super().deconstruct()
        return (name, "django.db.models.ForeignKey", args, kwargs)

    def formfield(self, **kwargs):
        if isinstance(kwargs["widget"], ForeignKeyRawIdWidget):
            kwargs["widget"].__class__ = CabinetFileRawIdWidget
        return super().formfield(**kwargs)



from django.contrib import admin
from django.template.defaultfilters import filesizeformat
from django.utils.html import format_html, format_html_join, mark_safe
from django.utils.translation import ugettext_lazy as _

from cabinet.base_admin import FileAdminBase
from cabinet.models import File


@admin.register(File)
class FileAdmin(FileAdminBase):
    list_display = ("admin_thumbnail", "admin_file_name", "admin_details")
    list_display_links = ("admin_thumbnail", "admin_file_name")

    def get_fieldsets(self, request, obj=None):
        if obj and obj.image_file.name:
            return [
                (
                    None,
                    {
                        "fields": (
                            "folder",
                            "image_file",
                            "image_ppoi",
                            "caption",
                            "image_alt_text",
                            "copyright",
                        )
                    },
                ),
                (_("Advanced"), {"fields": ("_overwrite",), "classes": ("collapse",)}),
            ]

        elif obj and obj.download_file.name:
            return [
                (None, {"fields": ("folder", "download_file", "caption", "copyright")}),
                (_("Advanced"), {"fields": ("_overwrite",), "classes": ("collapse",)}),
            ]

        else:
            return [
                (None, {"fields": ("folder", "caption", "copyright")}),
                (
                    _("Image"),
                    {"fields": ("image_file", "image_ppoi", "image_alt_text")},
                ),
                (_("Download"), {"fields": ("download_file",)}),
            ]

    def admin_thumbnail(self, instance):
        if instance.image_file.name:
            try:
                target = instance.image_file.process(["default", ("crop", (50, 50))])
                return format_html(
                    '<img src="{}" alt=""/>', instance.image_file.storage.url(target)
                )
            except Exception:
                return mark_safe('<span class="broken-image"></span>')
        elif instance.download_file.name:
            return format_html(
                '<span class="download download-{}">{}</span>',
                instance.download_type,
                instance.download_type.upper(),
            )
        return ""

    admin_thumbnail.short_description = ""

    def admin_file_name(self, instance):
        return format_html(
            "{} <small>({})</small>",
            instance.file_name,
            filesizeformat(instance.file_size),
        )

    admin_file_name.short_description = _("file name")

    def admin_details(self, instance):
        details = [instance.caption, instance.copyright]
        return format_html(
            "<small>{}</small>",
            format_html_join(mark_safe("<br>"), "{}", ((d,) for d in details if d)),
        )

    admin_details.short_description = _("details")
